﻿#include "pch.h"
#include "Key.h"

using namespace AnalogSynth;

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

DependencyProperty^ Key::isPressedProperty = 
    DependencyProperty::Register("IsPressed", 
                                 bool::typeid, 
                                 Key::typeid,
                                 ref new PropertyMetadata(false, 
                                    ref new PropertyChangedCallback(OnIsPressedChanged)));

DependencyProperty^ Key::keyNumberProperty = 
    DependencyProperty::Register("KeyNumber", 
                                 int::typeid, 
                                 Key::typeid,
                                 ref new PropertyMetadata(-1));

DependencyProperty^ Key::isFirstCProperty = 
    DependencyProperty::Register("IsFirstC", 
                                 bool::typeid, 
                                 Key::typeid,
                                 ref new PropertyMetadata(false));

Key::Key()
{
	DefaultStyleKey = "AnalogSynth.Key";
    pointerList = ref new Vector<unsigned int>();
}

void Key::OnPointerEntered(PointerRoutedEventArgs^ args)
{
    if (args->Pointer->IsInContact)
        AddToList(args->Pointer->PointerId);
}

void Key::OnPointerPressed(PointerRoutedEventArgs^ args)
{
    AddToList(args->Pointer->PointerId);
}

void Key::OnPointerReleased(PointerRoutedEventArgs^ args)
{
    RemoveFromList(args->Pointer->PointerId);
}

void Key::OnPointerExited(PointerRoutedEventArgs^ args)
{
    RemoveFromList(args->Pointer->PointerId);
}

void Key::AddToList(unsigned int id)
{
    unsigned int index;
    if (!pointerList->IndexOf(id, &index))
        pointerList->Append(id);

    CheckList();
}

void Key::RemoveFromList(unsigned int id)
{
    unsigned int index;
    if (pointerList->IndexOf(id, &index))
        pointerList->RemoveAt(index);

    CheckList();
}

void Key::CheckList()
{
    this->IsPressed = pointerList->Size > 0;
}

void Key::OnIsPressedChanged(DependencyObject^ obj, DependencyPropertyChangedEventArgs^ args)
{
    Key^ key = safe_cast<Key^>(obj);
    KeyPressedArgs^ keyPressedArgs = ref new KeyPressedArgs();
    keyPressedArgs->KeyNumber = key->KeyNumber;
    keyPressedArgs->IsPressed = key->IsPressed;
    key->IsPressedChanged(key, keyPressedArgs);
    VisualStateManager::GoToState(key, key->IsPressed ? "Pressed" : "Normal", false);
}
