﻿//
// MainPage.xaml.h
// Declaration of the MainPage class.
//

#pragma once

#include "MainPage.g.h"

// Most important components 
#include "SynthVoice.h"
#include "Patch.h"

// Control derivatives
#include "Key.h"
#include "Octave.xaml.h"

// Plain old data structures
#include "DynamicSynthVoiceParameters.h"
#include "DynamicFilterParameters.h"
#include "KeyPressedArgs.h"
#include "WaveformType.h"

// XAML support for displaying waveforms
#include "Waveform.h"

// Value converters
#include "TimeSliderValueConverter.h"
#include "CutoffFrequencyValueConverter.h"
#include "FilterEmphasisValueConverter.h"
#include "ToggleOnOffValueConverter.h"
#include "NoteCentValueConverter.h"

// Dialog boxes
#include "PatchSaveDialog.xaml.h"
#include "PatchLoadDialog.xaml.h"

namespace AnalogSynth
{
    public ref class MainPage sealed
    {
    private:
        Microsoft::WRL::ComPtr<IXAudio2> pXAudio2;
        IXAudio2MasteringVoice * pMasteringVoice;

        std::vector<SynthVoice *> allSynthVoices;
        std::vector<SynthVoice *> availableSynthVoices;
        std::map<int, SynthVoice *> playingSynthVoices;

        DynamicSynthVoiceParameters dynamicSynthVoiceParameters;
        Patch^ patch;

    public:
        MainPage();

    private:
        void OnDynamicPropertyChanged(Platform::Object^ sender, int args);
        void OnFilterEffectPropertyChanged(Platform::Object^ sender, int args);
        void OnKeyIsPressedChanged(Platform::Object^ sender, KeyPressedArgs^ args);
        void OnScrollViewerViewChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::ScrollViewerViewChangedEventArgs^ args);
        void OnSaveButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args);
        void OnLoadButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args);
        void OnVolumeSliderValueChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs^ args);
    };
}
